@@ -371,6 +371,99 @@ function select_by_number_gender (phrase,
 Ts.setcall("по-роду-броју", select_by_number_gender);
 // "Delete $[callname %1 this thisa thiso thees theesa theeso] %1?"
 
+// ------------------------------
+// Select one the form according to the case and gender of another argument.
+function select_by_case_gender (gcase, gphrase, phrase) // plurals
+{
+    var gender = getprop(gphrase, "_род");
+    return getprop(phrase, gcase + "-" + gender);
+}
+Ts.setcall("по-падежу-роду", select_by_case_gender);
+// "Delete $[callname case %2 %1] [case %2]?"
+
+// ------------------------------
+// Object to query whether a character is one of expected letters.
+letter_arr = (""
+    + "абвгдђежзијклљмнњопрстћуфхцчџшАБВГДЂЕЖЗИЈКЛЉМНЊОПРСТЋУФХЦЧЏШ"
+    + "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
+).split("");
+letter = {};
+for (var i = 0; i < letter_arr.length; ++i) {
+    letter[letter_arr[i]] = 1;
+}
+
+// ------------------------------
+// Split phrase into words and intersections,
+// where words are taken as contiguous segments of letters.
+// The return value is a tuple of arrays of words and intersections.
+// There is always one more of intersections than words, so that
+// the original phrase can be reassembled as
+// intrs[0] + words[0] + ... + intrs[n - 2] + words[n - 2] + intrs[n - 1].
+function split_phrase (phrase)
+{
+    intrs = [];
+    words = [];
+
+    var i = 0;
+    while (i < phrase.length) {
+        var i1 = i;
+        while (i1 < phrase.length && !letter[phrase[i1]]) {
+            ++i1;
+        }
+        intrs.push(phrase.substring(i, i1));
+        var i2 = i1;
+        while (i2 < phrase.length && letter[phrase[i2]]) {
+            ++i2;
+        }
+        if (i2 > i1) {
+            words.push(phrase.substring(i1, i2));
+            if (i2 == phrase.length) {
+                intrs.push("");
+            }
+        }
+        i = i2;
+    }
+
+    return [words, intrs];
+}
+
+// ------------------------------
+// Apply call to each word in the phrase.
+// The call must accept the word as the last argument.
+function apply_to_word (/* phrase, callname, ...args before word... */)
+{
+    if (arguments.length < 2)
+        throw Error("Applicator takes at least the phrase and the call name.");
+
+    phrase = arguments[0];
+    calln = arguments[1];
+    cargs = [calln];
+    for (var i = 2; i < arguments.length; ++i) {
+        cargs.push(arguments[i]);
+    }
+
+    lst = split_phrase(phrase);
+    words = lst[0];
+    intrs = lst[1];
+
+    nwords = [];
+    for (var i = 0; i < words.length; ++i) {
+        cargs.push(words[i]);
+        nwords.push(Ts.acall.apply(Ts, cargs));
+        cargs.pop();
+    }
+
+    str = ""
+    for (var i = 0; i < nwords.length; ++i) {
+        str += intrs[i] + nwords[i];
+    }
+    str += intrs[nwords.length];
+
+    return str;
+}
+Ts.setcall("на-реч", apply_to_word);
+// "Repeat until $[callname casecall %1], and on this date..."
+
 // ------------------------------
 // Decline person's name into given case.
 // Parse name into first and last name, determine gender according to
@@ -491,3 +584,4 @@ Ts.setcall("стил-према-фонту", match_style_to_font);
 // Property maps needed for all apps.
 Ts.loadProps("apps");
 Ts.loadProps("places");
+Ts.loadProps("calendar");
